	DEVICE ZXSPECTRUM48

;Interrupt based clock, v2, (C) George Chirtoaca, August 2007.
;It uses the trick explained below to minimize the RAM usage. This doesn't work with IF1 because of paging...Use the first version in this case.

;Taken from http://www.nvg.ntnu.no/sinclair/faq/tech_z80.html#INTERRUPTS
;A useful but not so much used trick on the Spectrum is to make the table contain FF's (or use the ROM for this) and put a byte 18 hex, the opcode for JR, at FFFF.
;The first byte of the ROM is a DI, F3 hex, so the JR will jump to FFF4, where a long JP to the actual interrupt routine is put.

ISR_ADDR		equ 65000
SCR_OFF			equ 16384 + 32 - 8	;screen offset address for display
FNT_TBL_ADDR	equ 23606			;font table pointer system variable
TICS_PER_SEC	equ 50				;number of tics per second
JP_OPCODE		equ $C3
JR_OPCODE		equ $18
ROM_INT_HANDLER	equ 056
ROM_INT_VEC		equ $3B
ROM_RET			equ $52
JP_ADDR			equ $FFF4


;The setup part
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	org ISR_ADDR - (EndLdr - start)
start:
	ld de, ISR_ADDR
	ld ix, JP_ADDR
	ld (ix), JP_OPCODE
	ld (ix + 1), e
	ld (ix + 2), d
	ld a, JR_OPCODE
	ld ($FFFF), a

	;move int routine to proper address
	call ROM_RET
Overhead:
	dec sp
	dec sp
	pop hl
	ld bc, StartIntRoutine - Overhead
	add hl, bc
	ld bc, EndIntRoutine - IntRoutine
	ldir

	;set interrupt
	di
	ld a, ROM_INT_VEC
	ld i, a
	im	2
	ei

	ret
EndLdr:


;The interrupt handler part
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

StartIntRoutine:

	org ISR_ADDR
IntRoutine:

	di
	push af
	push hl
	push bc
	push de
	push ix

	ld a, (TickCount)
	inc a
	cp TICS_PER_SEC
	jr nz, Clean

	call Payload	

	xor a
Clean:
	ld (TickCount), a
	
	call	Print
	
	pop ix
	pop de
	pop bc
	pop hl
	pop af
	jp ROM_INT_HANDLER		;this will re-enable interupts



Payload:	
	ld a, (SecCount)
	inc a
	ld (SecCount), a
	cp 60
	jr nz, Print
	xor a
	ld (SecCount), a

	ld a, (MinCount)
	inc a
	ld (MinCount), a
	cp 60
	jr nz, Print
	xor a
	ld (MinCount), a

	ld a, (HourCount)
	inc a
	ld (HourCount), a
	cp 24
	jr nz, Print
	xor a
	ld (HourCount), a

Print:	
	ld	hl, 16384+6144+32-8
	ld	a, (hl)	
	or	%01000000
	ld	b, 8
FillAttr:	
	ld	(hl), a
	inc	hl
	djnz FillAttr

	ld a, (HourCount)
	ld ix, SCR_OFF
	call N2DEC
	ld a, ':'
	ld ix, SCR_OFF + 2
	call PrintChar
	ld a, (MinCount)
	ld ix, SCR_OFF + 3
	call N2DEC
	ld a, ':'
	ld ix, SCR_OFF + 5
	call PrintChar
	ld a, (SecCount)
	ld ix, SCR_OFF + 6
	call N2DEC
	ret

;IN: IX = address, A = char
PrintChar:
	;find font address
	ld de, (FNT_TBL_ADDR)
	inc d
	sub ' '
	ld h, 0
	ld l, a
	add hl, hl
	add hl, hl
	add hl, hl
	add hl, de
	ld  de, 256
	ld  b, 8

PrintLine:
	ld a, (hl)
	ld (ix), a
	add ix, de
	inc hl
	djnz PrintLine

	ret


;IN : A = number
;OUT: DE = 2 digit representation of A
N2DEC:
	ld l, a
	ld h, 0
	ld bc, -10
	call DigitLoop
	ld e, a
	ld bc, -1
	call DigitLoop
	ld d, a

PrintNo:
	ld a, e
	push de
	call PrintChar
	pop de
	ld a, d
	ld  bc, -256*8 + 1
	add ix, bc
	call PrintChar
	ret


DigitLoop:
	ld	a, '0' - 1
DivNrLoop:
	inc	a			;increase reminder
	add	hl, bc		;substract divizor
	jr	c, DivNrLoop	;still dividing?
	sbc	hl, bc		;nope, restore
	ret

TickCount	defb 0
SecCount	defb 0
MinCount	defb 0
HourCount	defb 0
EndIntRoutine:

	savebin "clock2.bin", start, EndIntRoutine - start